@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.40';
+our $VERSION = '0.45';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -18,8 +18,8 @@ my $need_transpose = 0;
 my $inicial_price;
 my $iter_count_global = 0;
 my $iter_count_local  = 0;
-my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
-my ( %assignned_object, %assignned_person, %objects_desired_by_this, %price_object );
+my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size, $epsilon_scaling );
+my ( %assignned_object, %assignned_person, %price_object, %objects_desired_by_this );
 my %index_correlation;
 
 sub auction { #							=> default values
@@ -91,12 +91,13 @@ sub auction { #							=> default values
    
 		%assignned_object = ();
 		%assignned_person = ();
+		$epsilon_scaling++;
 		$iter_count_local = 0;
 
 		while ( (scalar keys %assignned_person) < $max_size ){ # while there is at least one element not assigned.
          
 			$iter_count_global++;
-			$iter_count_local++;
+			$iter_count_local++;		
 			
 			auctionRound( \@matrix, $epsilon, $args{verbose} );
 		 
@@ -115,8 +116,7 @@ sub auction { #							=> default values
 						if ( defined $assignned_person{$i} and $j == $assignned_person{$i} ){ print "**"; } else{ print "  "; }
 					}
 					if ($i >= 0){ print "]\n"; } else{ print "\n\n"; }
-				}
-				print "\n";		
+				}		
 			}		 
 		}
 		
@@ -161,7 +161,7 @@ sub auction { #							=> default values
 	}}   
    
 	if ( $args{verbose} >= 8 ){
-		printf "\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
+		printf "\n\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
 	}   
 	print_screen_messages( \@matrix, \@matrix_index, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
        
@@ -339,6 +339,7 @@ sub print_screen_messages {
       print "\n Pairs (in ascending order of matrix values):\n"; 
 
 	  my $sum_matrix_value = 0;
+	  my $sum_spaces = 2 * $matrix_spaces - 1;
       foreach my $matrix_value ( sort { $a <=> $b }  keys %orderly_solution ){
       foreach my $k ( sort { $a <=> $b } keys %{$orderly_solution{$matrix_value}} ){
      
@@ -349,8 +350,8 @@ sub print_screen_messages {
 	  
 		my $weight = ( defined $matrix_input[$index_array1] and defined $matrix_input[$index_array1]->[$index_array2] ) ? sprintf( "%${matrix_spaces}.${decimals}f", $matrix_value ) : ' ' x $matrix_spaces ;
 	  
-		printf( "   indices ( %${index_length}d, %${index_length}d ), matrix value = $weight ; sum of values = %${matrix_spaces}.${decimals}f \n", $index_array1, $index_array2, $sum_matrix_value );
-      }}	  
+		printf( "   indices ( %${index_length}d, %${index_length}d ), matrix value = $weight ; sum of values = %${sum_spaces}.${decimals}f \n", $index_array1, $index_array2, $sum_matrix_value );
+      }}
    }
 
 }
@@ -406,10 +407,10 @@ sub get_matrix_info {
 sub auctionRound {
 	my ( $matrix_ref, $epsilon, $verbose ) = @_;
 	my @matrix = @$matrix_ref;
-	my %info; my %seen_bid;
+	my ( %info, %seen_bid );
 
 	if ( $verbose >= 8 ){
-		print " Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
+		print "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
 			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
@@ -438,11 +439,10 @@ sub auctionRound {
 			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
 			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
 			my ( $bid01ForPersonI, $bid02ForPersonI );
+			my %current_value;
             
-			# "The set of objects to which each person i has been assigned in the last few assignments does not change much as the algorithm proceeds."
-			# Each person is assigned to just a relatively small number of objects during the execution of the auctionRound.
-			# My implementation is different from Libor Bus and Pavel Tvrdik: 'Towards auction algorithms for large dense assignment problems' (see reference 2).
-			# My working sets @objects_with_greater_benefits are updated dynamically by $objects_desired_by_this{$person} considering the price update of objects.					
+			# The working sets @objects_with_greater_benefits are updated dynamically by $objects_desired_by_this{$person} considering the price update of objects.
+			# For each person, the Current Value $curVal of the most desired objects contained in @objects_with_greater_benefits tend to converge to a specific value.
 			
 			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b }			
 			my @updated_price = grep { $objects_desired_by_this{$person}{$_} == $price_object{$_} } @objects_with_greater_benefits;
@@ -481,7 +481,7 @@ sub auctionRound {
 				}
 
 				if ( $verbose >= 8 ){
-					printf " personI = %3s ; objectJ = %3s ; \$curVal %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
+					printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
 				}				
 			}
 			
@@ -495,17 +495,17 @@ sub auctionRound {
 				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
 				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
 			
-				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );							
-				
-				if ( @updated_price == 0 ){ # all prices are outdated
+				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );				
+					
+				if ( @updated_price == 0 ){ # if all prices are outdated
 				
 					$objects_desired_by_this{$person}{$_} = $price_object{$_} for ( @objects_with_greater_benefits ); # update prices for all objects in the list
-
-					delete $objects_desired_by_this{$person} if ( @objects_with_greater_benefits > 15 ) ; # delete old list
+					
+					delete $objects_desired_by_this{$person} if ( @objects_with_greater_benefits > 6 + $epsilon_scaling ) ; # delete old list
 
 					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $price_object{$Opt01ObjForPersonI}; # information about the most desired objects
 					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $price_object{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
-					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $price_object{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);
+					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $price_object{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
 				}
 			}
 			
@@ -514,7 +514,7 @@ sub auctionRound {
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
 
-				printf "<> PersonI = %3s ; objects desired by this person (old list) = (@old_list) ; objects whose prices were not updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @updated_price;
+				printf "<> PersonI = %3s ; objects desired by this person (old list) = (@old_list) ; objects whose prices are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @updated_price;
 				printf "<> PersonI = %3s ; objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) \n", $person if ( defined $Opt03ObjForPersonI );
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
@@ -555,8 +555,8 @@ sub auctionRound {
 		last;  # next object, choose only the highest bid for each object
 	}}
 	
-	if ( $verbose >= 8 ){
-		print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
+	if ( $verbose >= 9 ){
+		print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; epsilon_scaling = $epsilon_scaling ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
 			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
@@ -797,7 +797,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 03, 2018
+	April 5, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
